%% Kyoto University, Statistical Machine Learning - Lecture 4 scripts and data
% Marco Cuturi, 2011
set(0,'defaulttextinterpreter','latex');
% Getting the data
% Obtained from http://www.amstat.org/publications/jse/datasets/body.dat
% See http://www.amstat.org/publications/jse/v11n2/datasets.heinz.html

load body.dat
% Body has now 25 columns and 507 observations. For the purpose of this
% lecture, we are only interested in the last 3 measurements: weight (kg)
% height (cm) and gender (1-M , 0-F)

body=body(:,end-2:end);

% compute density for illustration purposes. We will assume this is the
% "real" density.

% indices corresponding to males and females
IndM= (body(:,3)==1);
IndF= (body(:,3)==0);

% ratio of counts of M/F
MF_rat= sum(IndM+1-1)/sum(IndF+1-1);
p_M=sum(IndM+1-1)/size(body,1);


%%%%%%%%%%%%%%%%
% compute the Heights of Males or Females HM,HF, and plot an estimated density.

[HM,XHM]= ksdensity(body(IndM,2));
[HF,XHF]= ksdensity(body(IndF,2));

% to make sure density estimates are computed in the same points.
XH=sort(unique([XHM,XHF]));
XHM=XH;
XHF=XH;
HM= ksdensity(body(IndM,2),XH);
HF= ksdensity(body(IndF,2),XH);


% housekeeping
%close all;
rand('state',0);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Hoeffding Inequality Demo
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

FUNCTION=@(x,threshold) .5*sign(x-threshold)+.5;
% compute true risk10
TRUE_RISK= @(threshold) trapz(XH,(FUNCTION(XH,threshold)~=1).*(HM*p_M)+(FUNCTION(XH,threshold)~=0).*(HF*(1-p_M)) );




% compute density estimates to draw random samples.
Distmales=fitdist(body(IndM,2),'kernel');
Distfemales=fitdist(body(IndF,2),'kernel');

% number of samples to test Hoeffding's inequality
N=300;


threshold=165;
TRUE_RISK_=TRUE_RISK(threshold);
EMPIRICAL_RISK=zeros(1,N);
RANDNOISE=rand(1,N);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% PREPARE PLOTS
figure(2);
subplot(2,2,1);
cla reset;
hold on;
plot(XHM,HM*p_M,'b-','LineWidth',2);
plot(XHF,HF*(1-p_M),'r-','LineWidth',2);
legend({'p(X,Y=1)','p(X,Y=0)'})
set(gca,'FontSize',20);
title('True Distribution','FontSize',16);
axis tight;


subplot(2,2,3);
cla reset;
hold on;
h=hline_(TRUE_RISK_,'b-',['True Risk : ',num2str(TRUE_RISK_)]);
xlim([.4,.6]);
set(gca,'xtick',[]);
ylim([0,0.7]);
set(gca,'FontSize',20);
hold off;

set(h,'LineWidth',2);
xlim([0,1]);
set(gca,'FontSize',20);
hold off;

subplot(2,2,2);
FF= @(x) FUNCTION(x,threshold);
ezplot(FF,[min(XH),max(XH)]);
hhh=title(['Considered function $f$: Threshold ',num2str(threshold),' cm'],'FontSize',16);
set(hhh,'Interpreter','latex');
set(gca,'FontSize',20);
vline(threshold);

subplot(2,2,4)
hold on
h=ezplot(TRUE_RISK,[140,190]);
set(h, 'Color', 'b');
set(h,'Linewidth',3);
xlabel('Threshold Level');
ylabel('Risk')
title('Risk','FontSize',16);
legend('True Risk');
ylim([0,0.7]);
vline(threshold);
set(gca,'FontSize',20);
hold off;    
pause();
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




for samples=1:N,
    % number of samples for each testok
    n=30;
    Z=zeros(n,2);
    
    
    if samples<6
        subplot(2,2,2);
    FF= @(x) FUNCTION(x,threshold);
    ezplot(FF,[min(XH),max(XH)]);
    hhh=title(['Considered function $f$: Threshold ',num2str(threshold),' cm'],'FontSize',16);
    set(hhh,'Interpreter','latex');
    set(gca,'FontSize',20);
    vline(threshold);
    end

    
    %% PLOTTING stuff
    figure(2);
    subplot(2,2,1);
    cla reset;
    hold on;
    plot(XHM,HM*p_M,'b--','LineWidth',1);
    plot(XHF,HF*(1-p_M),'r--','LineWidth',1);
    set(gca,'FontSize',20);
    axis tight;
    Height=0.015;
    title('Empirical Sample','FontSize',16);
    
    for ind=1:n,
        % first decide if sampling from male or female distribution
        if rand()<p_M,            
            Z(ind,:)=[random(Distmales),1];
            subplot(2,2,1);
            hold on;
            h=scatter(Z(ind,1),Height*(1+.1*(rand()-.5)),60,'blue','o');
            set(h,'LineWidth',1.5);
            hold off;
            subplot(2,2,2);
            hold on;
            scatter(Z(ind,1),1,60,'blue','o');
            hold off;
        else
            subplot(2,2,1);
            hold on;
            Z(ind,:)=[random(Distfemales),0];
            h=scatter(Z(ind,1),Height*(1+.1*(rand()-.5)),60,'red','d');
            set(h,'LineWidth',1.5);
            hold off;
            subplot(2,2,2);
            hold on;
            scatter(Z(ind,1),0,60,'red','d');
            hold off;
        end
    end
    hold off;
    if samples<=10,
        pause();
    end
    EMPIRICAL_RISK(samples)=sum(FUNCTION(Z(:,1),threshold)~=Z(:,2))/n;
    
    EMPIRICAL_RISK_FUNCTION= @(threshold) sum(FUNCTION(Z(:,1),threshold)~=Z(:,2))/n;
    subplot(2,2,4)
    hold on
    h=ezplot(EMPIRICAL_RISK_FUNCTION,[140,190]);
    set(h, 'Color', 'r');
    set(h,'Linewidth',1);
    title('Risk','FontSize',16);
    ylim([0,0.7]);

    
    %% SECOND PLOTTING stuff
    subplot(2,2,3);
    %cla reset;
    hold on;
    h=scatter(0.5*ones(1,samples)+.1*(RANDNOISE(1:samples)-.5),EMPIRICAL_RISK(1:samples),120,'black','x');        
    set(h,'LineWidth',1);
    xlim([.4,.6]);
    set(gca,'xtick',[]);
    %h=hline(TRUE_RISK_,'b-',['True Risk : ',num2str(TRUE_RISK_)]);
    %set(h,'LineWidth',2);
    ylim([0,0.7]);
    set(gca,'FontSize',20);
    hold off;
    
    if samples<6,
        for iii=1:samples,
            text(0.5+.1*(RANDNOISE(iii)-.5),EMPIRICAL_RISK(iii)+0.05,num2str(EMPIRICAL_RISK(iii)));
        end
    end
    %% THIRD PLOTTING STUFF
    if samples>5,
        subplot(2,2,2);
        cla reset;
        hold on;
        [NN,xout]=hist(abs(EMPIRICAL_RISK(1:samples)-TRUE_RISK_),0:0.025:0.6);
        bar(xout,NN/sum(NN));
        axis([0,.6,0,1]);
        hold off;
        title('Frequency of deviations','FontSize',16);
        pause(.01);drawnow;
        set(gca,'FontSize',20);
    else
        pause();
    end
end












